=== added directory 'plugin/information_engine'
=== added file 'plugin/information_engine/information_engine.cc'
--- plugin/information_engine/information_engine.cc	1970-01-01 00:00:00 +0000
+++ plugin/information_engine/information_engine.cc	2009-10-15 00:22:33 +0000
@@ -0,0 +1,186 @@
+/* Copyright (C) 2005 MySQL AB
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
+#include "information_engine.h"
+#include <drizzled/table.h>
+
+#include <string>
+
+using namespace std;
+
+static const string engine_name("INFORMATION_ENGINE");
+
+/*****************************************************************************
+** INFORMATION_ENGINE tables
+*****************************************************************************/
+
+InformationCursor::InformationCursor(drizzled::plugin::StorageEngine *engine_arg,
+                                     TableShare *table_arg) :
+  handler(engine_arg, table_arg)
+{}
+
+uint32_t InformationCursor::index_flags(uint32_t, uint32_t, bool) const
+{
+  return 0;
+}
+
+int InformationCursor::open(const char *name, int, uint32_t)
+{
+  InformationShare *shareable;
+
+  if (!(shareable= InformationShare::get(name)))
+    return(HA_ERR_OUT_OF_MEM);
+
+  thr_lock_data_init(&shareable->lock, &lock, NULL);
+
+  return 0;
+}
+
+int InformationCursor::close(void)
+{
+  InformationShare::free(share);
+
+  return 0;
+}
+
+int InformationEngine::createTableImplementation(Session*, const char *,
+                                                 Table *, HA_CREATE_INFO *,
+                                                 drizzled::message::Table*)
+{
+  return 1;
+}
+
+int InformationEngine::deleteTableImplementation(Session*, const string)
+{
+  return 0;
+}
+
+int InformationCursor::write_row(unsigned char *)
+{
+  return(table->next_number_field ? update_auto_increment() : 0);
+}
+
+int InformationCursor::rnd_init(bool)
+{
+  return 0;
+}
+
+
+int InformationCursor::rnd_next(unsigned char *)
+{
+  return HA_ERR_END_OF_FILE;
+}
+
+
+int InformationCursor::rnd_pos(unsigned char *, unsigned char *)
+{
+  assert(1);
+
+  return 0;
+}
+
+
+void InformationCursor::position(const unsigned char *)
+{
+  assert(1);
+}
+
+
+int InformationCursor::info(uint32_t flag)
+{
+  memset(&stats, 0, sizeof(stats));
+  if (flag & HA_STATUS_AUTO)
+    stats.auto_increment_value= 1;
+  return(0);
+}
+
+int InformationCursor::external_lock(Session *, int)
+{
+  return 0;
+}
+
+
+THR_LOCK_DATA **InformationCursor::store_lock(Session *session,
+                                         THR_LOCK_DATA **to,
+                                         enum thr_lock_type lock_type)
+{
+  if (lock_type != TL_IGNORE && lock.type == TL_UNLOCK)
+  {
+    /*
+      Here is where we get into the guts of a row level lock.
+      If TL_UNLOCK is set
+      If we are not doing a LOCK Table or DISCARD/IMPORT
+      TABLESPACE, then allow multiple writers
+    */
+
+    if ((lock_type >= TL_WRITE_CONCURRENT_INSERT &&
+         lock_type <= TL_WRITE) && !session_tablespace_op(session))
+      lock_type = TL_WRITE_ALLOW_WRITE;
+
+    /*
+      In queries of type INSERT INTO t1 SELECT ... FROM t2 ...
+      MySQL would use the lock TL_READ_NO_INSERT on t2, and that
+      would conflict with TL_WRITE_ALLOW_WRITE, blocking all inserts
+      to t2. Convert the lock to a normal read lock to allow
+      concurrent inserts to t2.
+    */
+
+    if (lock_type == TL_READ_NO_INSERT)
+      lock_type = TL_READ;
+
+    lock.type= lock_type;
+  }
+  *to++= &lock;
+
+  return to;
+}
+
+static drizzled::plugin::StorageEngine *information_engine= NULL;
+
+static int init(drizzled::plugin::Registry &registry)
+{
+
+  information_engine= new InformationEngine(engine_name);
+  registry.add(information_engine);
+  
+  InformationShare::start();
+
+  return 0;
+}
+
+static int finalize(drizzled::plugin::Registry &registry)
+{
+  registry.remove(information_engine);
+  delete information_engine;
+
+  InformationShare::stop();
+
+  return 0;
+}
+
+drizzle_declare_plugin(information_engine)
+{
+  "INFORMATION_ENGINE",
+  "1.0",
+  "Sun Microsystems ala Brian Aker",
+  "Engine which provides information schema tables",
+  PLUGIN_LICENSE_GPL,
+  init,     /* Plugin Init */
+  finalize,     /* Plugin Deinit */
+  NULL,               /* status variables */
+  NULL,               /* system variables */
+  NULL                /* config options   */
+}
+drizzle_declare_plugin_end;

=== added file 'plugin/information_engine/information_engine.h'
--- plugin/information_engine/information_engine.h	1970-01-01 00:00:00 +0000
+++ plugin/information_engine/information_engine.h	2009-10-15 00:22:33 +0000
@@ -0,0 +1,108 @@
+/* Copyright (C) 2005 MySQL AB
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
+
+#ifndef PLUGIN_INFORMATION_ENGINE_HA_INFORMATION_ENGINE_H
+#define PLUGIN_INFORMATION_ENGINE_HA_INFORMATION_ENGINE_H
+
+#include <drizzled/server_includes.h>
+#include <drizzled/handler.h>
+#include <mysys/thr_lock.h>
+#include "information_share.h"
+
+
+/*
+  Class definition for the information engine
+*/
+class InformationCursor: public handler
+{
+  THR_LOCK_DATA lock;      /* MySQL lock */
+  InformationShare *share;
+
+public:
+  InformationCursor(drizzled::plugin::StorageEngine *engine, TableShare *table_arg);
+  ~InformationCursor()
+  {}
+
+  /*
+    The name of the index type that will be used for display
+    don't implement this method unless you really have indexes
+  */
+  uint64_t table_flags() const
+  {
+    return 0;
+  }
+  uint32_t index_flags(uint32_t inx, uint32_t part, bool all_parts) const;
+  /* The following defines can be increased if necessary */
+  int open(const char *name, int mode, uint32_t test_if_locked);
+  int close(void);
+  int write_row(unsigned char * buf);
+  int rnd_init(bool scan);
+  int rnd_next(unsigned char *buf);
+  int rnd_pos(unsigned char * buf, unsigned char *pos);
+  void position(const unsigned char *record);
+  int info(uint32_t flag);
+  int external_lock(Session *session, int lock_type);
+  THR_LOCK_DATA **store_lock(Session *session,
+                             THR_LOCK_DATA **to,
+                             enum thr_lock_type lock_type);
+};
+
+static const char *InformationEngine_exts[] = {
+  NULL
+};
+
+class InformationEngineNameIterator: public drizzled::plugin::TableNameIteratorImplementation
+{
+public:
+  InformationEngineNameIterator(const std::string &database)
+    : drizzled::plugin::TableNameIteratorImplementation(database)
+    {};
+
+  int next(std::string *)
+  {
+    return 1;
+  }
+
+};
+
+class InformationEngine : public drizzled::plugin::StorageEngine
+{
+public:
+  InformationEngine(const string &name_arg)
+   : drizzled::plugin::StorageEngine(name_arg,
+                                     HTON_FILE_BASED
+                                      | HTON_HAS_DATA_DICTIONARY) {}
+  virtual handler *create(TableShare *table, MEM_ROOT *mem_root)
+  {
+    return new (mem_root) InformationCursor(this, table);
+  }
+
+  const char **bas_ext() const {
+    return InformationEngine_exts;
+  }
+
+  int createTableImplementation(Session*, const char *, Table *,
+                                HA_CREATE_INFO *, drizzled::message::Table*);
+
+  int deleteTableImplementation(Session*, const string table_name); 
+
+  drizzled::plugin::TableNameIteratorImplementation* tableNameIterator(const std::string &database)
+  {
+    return new InformationEngineNameIterator(database);
+  }
+};
+
+#endif /* PLUGIN_INFORMATION_ENGINE_HA_INFORMATION_ENGINE_H */

=== added file 'plugin/information_engine/information_share.cc'
--- plugin/information_engine/information_share.cc	1970-01-01 00:00:00 +0000
+++ plugin/information_engine/information_share.cc	2009-10-15 00:22:33 +0000
@@ -0,0 +1,72 @@
+/* Copyright (C) 2009 Sun Microsystems
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
+#include "information_share.h"
+
+#include <map>
+#include <utility>
+
+using namespace std;
+
+typedef map<const char *, InformationShare* > mapType;
+static mapType open_tables;
+
+pthread_mutex_t share_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+InformationShare *InformationShare::get(const char *table_name)
+{
+  InformationShare *share;
+  mapType::iterator iter;
+  uint32_t length;
+
+  length= (uint) strlen(table_name);
+  pthread_mutex_lock(&share_mutex);
+
+  iter = open_tables.find(table_name);
+
+  if (iter!= open_tables.end())
+  {
+    share= (*iter).second;
+    share->inc();
+    return share;
+  }
+
+  share= new (nothrow) InformationShare(table_name);
+
+  pthread_mutex_unlock(&share_mutex);
+
+  return share;
+}
+
+void InformationShare::start(void)
+{
+  /* NULL */
+}
+
+void InformationShare::stop(void)
+{
+  pthread_mutex_destroy(&share_mutex);
+  open_tables.clear();
+}
+
+void InformationShare::free(InformationShare *share)
+{
+  pthread_mutex_lock(&share_mutex);
+
+  if (share->dec() == 0)
+    open_tables.erase(share->name.c_str());
+  pthread_mutex_unlock(&share_mutex);
+}
+

=== added file 'plugin/information_engine/information_share.h'
--- plugin/information_engine/information_share.h	1970-01-01 00:00:00 +0000
+++ plugin/information_engine/information_share.h	2009-10-15 00:22:33 +0000
@@ -0,0 +1,57 @@
+/* Copyright (C) 2009 Sun Microsystems
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+
+
+#ifndef PLUGIN_INFORMATION_ENGINE_INFORMATION_SOURCE_H
+#define PLUGIN_INFORMATION_ENGINE_INFORMATION_SOURCE_H
+
+#include <drizzled/server_includes.h>
+
+/*
+  Shared class for correct LOCK operation
+  TODO -> Fix to never remove/etc. We could generate all of these in startup if we wanted to.
+  Tracking count? I'm not sure that is needed at all. We could possibly make this a member of
+  engine as well (should we just hide the share detail?)
+*/
+
+class InformationCursor;
+
+class InformationShare {
+  uint32_t count;
+  string name;
+
+public:
+  InformationShare(const char *arg) :
+    count(1),
+    name(arg)
+  {
+    thr_lock_init(&lock);
+  };
+  ~InformationShare() 
+  {
+    thr_lock_delete(&lock);
+  }
+
+  void inc(void) { count++; }
+  uint32_t dec(void) { return --count; }
+
+  static InformationShare *get(const char *table_name);
+  static void free(InformationShare *share);
+  static void start(void);
+  static void stop(void);
+  THR_LOCK lock;
+};
+
+#endif /* STORAGE_INFORMATION_ENGINE_INFORMATION_SOURCE_H */

=== added file 'plugin/information_engine/plugin.ini'
--- plugin/information_engine/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/information_engine/plugin.ini	2009-10-15 00:22:33 +0000
@@ -0,0 +1,7 @@
+[plugin]
+name=information_engine
+title=Information Engine
+description=Information Engine for Information Schema
+load_by_default=yes
+sources=information_engine.cc information_share.cc
+headers=information_engine.h information_share.h

